import matplotlib.pyplot as plt
from typing import List

def EMI(principal :int,rate :float,tenure :int) ->None:
    
    """
    Calculates and visualises Equated Monthly Interest(EMI)

    Parameters:
      principal(float) : principal amount 
      rate(float) : Rate of interest(in percentage)
      tenure(int) : number of years for which EMI has to be calculated

    Return:
      The function has void return type.

    Example:
       EMI(500000,10.5,2)

    """

    rate : float= rate/(12*100) #rate of interest 
    tenure : float= tenure * 12 # no of months
    emi :float = round((principal * rate * pow(1+rate,tenure))/(pow(1+rate,tenure)-1),0)
    print("Loan EMI: ",emi)
    
    # total amount to be paid after the mentioned tenure
    total_amt : float = emi * tenure 
    interest : float= round((total_amt - principal)*100/total_amt,0)
    loan_amt: float= principal * 100/total_amt
    print("Total Payment: ",total_amt)
    print("Total Interest Payable: ",total_amt - principal)
    
    # plotting interest and principal in a pie chart
    fig1, (ax1,ax2)= plt.subplots(2)
    fig1.suptitle('EMI')
    fig2,(ax3)=plt.subplots()
    fig2.suptitle('Month-wise Details')
    x = [loan_amt,interest]
    mylabels = ("Principal Loan Amount","Interest")
    myexplode = [0,0.2]
    ax1.pie(x,autopct ='%.1f',labels = mylabels,explode = myexplode)
    ax2.legend()
    
    
    # required lists for plotting graph and table
    interest_data :List[float]=[]
    principal_data: List[float]=[]
    balance_data: List[float]=[]
    month_no : List[float]= []
    cell_text :List[float]=[]
    columns = ["Month_No","Principal","Interest","Balance"]
    prev_bal = principal
    
    # finding monthly principal and interest
    for i in range(tenure):
        month_interest = round(prev_bal*rate,0)
        month_principal = emi - month_interest
        prev_bal = prev_bal - month_principal
        
        interest_data.append(month_interest)
        principal_data.append(month_principal)
        balance_data.append(prev_bal)
        month_no.append(i+1)
        cell_text.append([i+1,month_principal,month_interest,prev_bal])
        
    # plotting bar graph
    
    ax2.bar(month_no, principal_data, width=0.35, label='Principal')
    ax2.bar(month_no, interest_data, width=0.35, bottom=principal_data,
       label='Interest')
    ax2.set_xlabel('Months')
    ax2.set_ylabel('EMI Payment')
    ax2.set_title('EMI Payment/month')
    ax2.legend()
    
    
    
    # Table creation
    
    ax3.set_axis_off()
    table = ax3.table(cellText= cell_text,colLabels=columns,loc='upper center',cellLoc='center')
    
    table.set_fontsize(10)
    table.scale(2,2)
    plt.tight_layout()
    plt.show()


def FD(Principle :int,Rate :float,Time :int,freq :str) ->None:
    
    
    """
    Calculates and Visualises Fixed Deposit(FD)
    
    Parameters:
     Principle(int) : FD Amount
     Rate(float) : Rate of Interest(in percentage)
     Time(int) : Number of years for which Fd is created
     freq(str) : The frequency in which compound interest should be calculated.
                 The values can be from the list: ("annually","quaterly","monthly")
     
    Returns:
     Return type: void
      
    Example:
     FD(50000,10.5,5,'annually')
     
     """
     
    #Required list for plotting Graph and pie chart
    fd_amount : int=Principle
    closing_balance : List[float]=[]
    interest_data : List[float]=[]
    year :List[float]=[]
    
    fig, (ax1,ax2,ax3)=plt.subplots(3)
    fig.suptitle('Fixed Deposit')
       
    if(freq== "annually"):

        for i in range(Time):
            a=Principle*(1+Rate/100)**1
            year.append(i+1)
            closing_balance.append(a)
            interest_data.append(a-Principle)
            Principle=a
            print("Interest Earned at the end of : ")
            print("Year {} is : {} ".format(i+1,round(interest_data[i])))
            
        #printing final data
        print("\n\n")
        print("Amount Deposited : ",fd_amount)
        print("Maturity Amount : ",round(closing_balance[-1]))
        print("Interest Earned : ",round(sum(interest_data)))
        
       
        # Pie chart
        x=[fd_amount,sum(interest_data)]
        mylabels=("FD amount","Interest Earned")
        myexplode=[0,0.2]
        ax1.pie(x,autopct ='%.1f',labels = mylabels,explode = myexplode,shadow=True)
        ax1.legend()
        ax1.set_title("Annually Cummulated")
        


        # Bar graph for Balance
        ax2.bar(year,closing_balance,color="#800000")
        ax2.set_title("FD Amount over the Year(Annually)")
        ax2.set_xlabel("Years")
        ax2.set_ylabel("Balance")
        
        

        # Bar graph for interest 
        ax3.bar(year,interest_data)
        ax3.set_title("Interest earned over the Year(Annually)")
        ax3.set_xlabel("Years")
        ax3.set_xlabel("Interest")

        plt.tight_layout()
        plt.show()




    elif (freq == "quaterly"):

        for i in range(Time):
            a=Principle*(1+Rate/400)**4
            year.append(i+1)
            closing_balance.append(a)
            interest_data.append(a-Principle)
            Principle=a
            print("Interest Earned at the end of : ")
            print("Year {} is : {} ".format(i+1,round(interest_data[i])))
            
        #printing final data
        print("\n\n")
        print("Amount Deposited : ",fd_amount)
        print("Maturity Amount : ",round(closing_balance[-1]))
        print("Interest Earned : ",round(sum(interest_data)))
     
        

        #Pie chart
        x=[fd_amount,sum(interest_data)]
        mylabels=("FD amount","Interest Earned")
        myexplode=[0,0.2]
        ax1.pie(x,autopct ='%.1f',labels = mylabels,explode = myexplode,shadow=True)
        ax1.legend()
        ax1.set_title("Quaterly Cummulated")
        

        
        # Bar graph for Balance
        ax2.bar(year,closing_balance,color="#800000")
        ax2.set_title("FD Amount over the Year(Quaterly)")
        ax2.set_xlabel("Years")
        ax2.set_ylabel("Balance")
        


        # Bar graph for interest 
        plt.bar(year,interest_data)
        ax3.set_title("Interest earned over the Year(Quaterly)")
        ax3.set_xlabel("Years")
        ax3.set_ylabel("Interest")
        plt.tight_layout()
        plt.show()
        



    elif (freq == "monthly"):
        for i in range(Time):
            a=Principle*(1+Rate/1200)**12
            year.append(i+1)
            closing_balance.append(a)
            interest_data.append(a-Principle)
            Principle=a

            print("Interest Earned at the end of : ")
            print("Year {} is : {}".format(i+1,round(interest_data[i])))
            
        #Printing final data
        print("\n\n")
        print("Amount Deposited : ",fd_amount)
        print("Maturity Amount : ",round(closing_balance[-1]))
        print("Interest Earned : ",round(sum(interest_data)))
        
       
        # Pie chart 
        x=[fd_amount,sum(interest_data)]
        mylabels=("FD amount","Interest Earned")
        myexplode=[0,0.2]
        ax1.pie(x,autopct ='%.1f',labels = mylabels,explode = myexplode,shadow=True)
        ax1.legend()
        ax1.set_title("Monthly Cummulated")
        

        # Bar graph for Balance
        ax2.bar(year,closing_balance,color="#800000")
        ax2.set_title("FD Amount over the Period(Monthly)")
        ax2.set_xlabel("Years")
        ax2.set_ylabel("Balance")
        


        # Bar graph for interest 
        ax3.bar(year,interest_data)
        ax3.set_title("Interest earned over the Year(monthly)")
        ax3.set_xlabel("Years")
        ax3.set_ylabel("Interest")
        plt.tight_layout()
        plt.show()



from typing import List
import matplotlib.pyplot as plt

def simple_interest(Principle :float,Rate :float,Time: int) ->None:

    """
     Calculates and Visualises Simple Interest

     Parameters:
      Principle(int) : Principle amount
      Rate(float) : Rate of Interest(in percentage)
      Time(int) : Time period(in years)

     Return:
      This function does not return anything.

     Example:
      simple_interest(5000,5.5,5)

    """

    #defining required lists
    Years : List[float]=[]
    SI : List[float]=[]
    Balance :List[float] =[]
    # Simple interest calculation along with graphical representation
    Deposit : float =Principle
    TI : float =Principle*Rate*Time/100
    I : float =Principle*Rate/100
    A : float =Principle+TI
    for i in range(Time):
        Principle=Principle+I
        Balance.append(Principle)
        Years.append(i+1)
        SI.append(I)
    print("Amount deposited is:",Deposit)
    print("Intrest earned at the end of each year is:",I)
    print("Total amount at the end of {} years (at maturity) is: {}".format(Time,A))

    #plotting the Balance V/S Years graph
    fig, (ax1,ax2,ax3) = plt.subplots(3)
    fig.suptitle('Simple Interest')
    ax1.plot(Years,Balance,color='#444444', linestyle='-',)
    ax1.legend()
    ax1.set_title('Simple Interest Graph (Balance v/s Years)')
    ax1.set_xlabel('Years')
    ax1.set_ylabel('Balance')

    # plotting the SI V/S Years graph
    ax2.plot(Years,SI,color='#444444', linestyle='--',)
    ax2.legend()
    ax2.set_title('Simple Interest Graph (SI v/s Years)')
    ax2.set_xlabel('Years')
    ax2.set_ylabel('Simple Interest')

    # Plotting the pie chart
    x=[]
    x.append(Deposit)
    x.append(TI)
    x.append(A)
    names=["Deposit","Interest","Total_Amount"]
    depexplode=[0,0,0]
    ax3.pie(x,labels=names,explode=depexplode,autopct='%1.1f%%', shadow=True)
    ax3.set_title('Ratios')

    plt.tight_layout()
    plt.show() 


def compound_interest(P :int,R :float,T :int,freq :str) -> None:
    
    
    """
     Calculates and visualises compound interest 

     Parameters:
        P(float) : Principal amount
        R(float) : Rate of interest (in percentage)
        T(int) : Time
        freq(str) : The frequency in which compound interest should be calculated. 
                    The values can be from the list: ('annually','semiannually','quaterly','monthly')

    Return:
        This function does not return anything.
    
    Example:
       compound_interest(20000,4.5,5,'quaterly')
        
    """


    # defining required lists
    interest : List[float] = []
    time : List[float] = []
    Amount: List[float] = []
    deposit: List[float]=P
    TI : int=0
    fq : int=1
    term : str=' '
    fig, (ax1,ax2)= plt.subplots(2)
    fig.suptitle('Compound Interest')
    
    # compound interest calculation along with graphical representation 
    if(freq=='monthly'):
        fq=12
        term='Month'
    elif (freq == 'annually'):
        fq=1
        term='Year'
    elif (freq == 'quaterly'):
        fq=4
        term='Quarter-Year'
    elif(freq == 'semiannually'):
        fq=2
        term='Semi-Year'
        
    for i in range(fq*T):
        I = P * R/(fq*100)
        P = P + I
        TI+=I
        print("Interest recieved in {} {} is {}".format(term,i+1,I))
        interest.append(I)
        time.append(i+1)
        Amount.append(P)
    print("In {} years you will have {} if interest is compounded {} ".format(T,P,freq))
    ax1.plot(time,Amount)
    ax1.set_xlabel(term)
    ax1.set_ylabel("Amount")
        
        
    x=[]
    x.append(deposit)
    x.append(TI)
    names=["Deposit","Interest"]
    depexplode=[0,0.2]
    ax2.pie(x,labels=names,explode=depexplode , shadow=True)
    ax2.set_title('Ratios')
    plt.tight_layout()
    plt.show() 



from typing import List, Union
import numpy as np
import pandas_datareader as pdr
import pandas as pd
import matplotlib.pyplot as plt


def rsi(symbol :str ,name :str, date :str) -> None :
    """
    Calculates and visualises the Relative Stock Index on a Stock of the company.

    Parameters:
        symbol(str) : Symbol of the company from  https://in.finance.yahoo.com/
        name(str) : Name of the company
        date(str) : start date of historical data in the format (YYYY,M,D)
    Returns:
        Return type: void  
    Example:
        rsi('GOOG','Google','2020,01,01')     

    """
    ticker : str = pdr.get_data_yahoo(symbol, date)
    delta : List[float]  = ticker['Close'].diff()
    up : int   = delta.clip(lower=0)
    down : int = -1*delta.clip(upper=0)
    ema_up : Union[bool,float]= up.ewm(com=13, adjust=False).mean()
    ema_down : Union[bool,float] = down.ewm(com=13, adjust=False).mean()
    rs : float = ema_up/ema_down
    ticker['RSI'] = 100 - (100/(1 + rs))
    ticker : list = ticker.iloc[14:]
    print(ticker)
    fig, (ax1, ax2) = plt.subplots(2)
    ax1.get_xaxis().set_visible(False)
    fig.suptitle(name)
    ticker['Close'].plot(ax=ax1)
    ax1.set_ylabel('Price ($)')
    ticker['RSI'].plot(ax=ax2)
    ax2.set_ylim(0,100)
    ax2.axhline(30, color='r', linestyle='--')
    ax2.axhline(70, color='r', linestyle='--')
    ax2.set_ylabel('RSI')
    plt.show()


def volatility(symbol :str, date :str) ->None:
    """
    Measures and visualizes the Volatility of a Stock by calculating the Average True Range(ATR)
    Parameters:
        symbol(str) : Symbol of the company from  https://in.finance.yahoo.com/
        date(str) : start date of historical data in the format (YYYY,M,D)        
    Returns:
        Return type: void
    Example:
        volatility('GOOG','2020,01,01')    

    """
    
    data : str = pdr.get_data_yahoo(symbol,date)
    data.head()
    high_low : Union[int,float]= data['High'] - data['Low']
    high_cp : List[float] = np.abs(data['High'] - data['Close'].shift())
    low_cp : List[float]= np.abs(data['Low'] - data['Close'].shift())
    df : List[str] = pd.concat([high_low, high_cp, low_cp], axis=1)
    true_range : float= np.max(df, axis=1)
    average_true_range : float= true_range.rolling(14).mean()
    average_true_range
    true_range.rolling(14).sum()/14
    
    fig, ax = plt.subplots()
    average_true_range.plot(ax=ax)
    ax2 : Union[bool,float]= data['Close'].plot(ax=ax, secondary_y=True, alpha=.3)
    ax.set_ylabel("ATR")
    ax2.set_ylabel("Price")
    plt.show()


import setuptools

with open("README.md","r",encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name="FinanceLib",
    version="0.0.2",
    description='A python library that consists of modules for different types of financial calculations and analysis.',
    long_description= long_description,
    long_description_content_type='text/markdown',
    author='Chahat Mittal,Harleen Kaur,Sristhi Sahoo',
    author_email='financeLib027@gmail.com',
    python_requires='>=3.7',
    url='https://github.com/sris-spec/FinanceLib.git',
    install_requires=['matplotlib','numpy','pandas_datareader','pandas','mypy'],
    license='MIT',
    package_dir = {"": "src"},
    packages = setuptools.find_packages(where = "src"),
    keywords =['python', 'Finance with python', 'interest', 'analysis', 'general banking' , 'visualising finance','stock','yahoo finance'],
    classifiers=[
        
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        'License :: OSI Approved :: MIT License',
        
    ],
    
)
